--[[

	BPDE API library

	Written by Bjrn Petter Kysnes (bepe86) for PSP OSi.

	You can contact the author (me) by mail (bk@cold-blade.com)

--]]


-- Define some constants used to all applications

RESOLUTION = {
	X = 480,
	Y = 272
}

-- Define the element "class" and it's methods

Element = {
	frame = 0,
	num = 0,
	width = { },
	height = { },
	x_pos = { },
	y_pos = { },
	parent = { },
	is_parent = { },
	refresh_timer = { },
	refresh_interval = { },
	render_function = { },
	last_rendered = { },
	enabled = { },
	canvas = { }
}

--[[
	Usage: Element:add{options} - note the { }instead of ( )

	Options: [x_pos (int)], [y_pos (int)], [width (int)], [height (int)], [parent (int)], [refresh_interval (int)], [enabled (int)], render_function (function (! without () )

--]]

function Element:add(options)
	self.num = self.num + 1
	self.parent[self.num] = options.parent or 0

	self.x_pos[self.num] = options.x_pos or 0
	self.y_pos[self.num] = options.y_pos or 0
	self.width[self.num] = options.width or RESOLUTION.X
	self.height[self.num] = options.height or RESOLUTION.Y
	if self.parent[self.num] ~= 0 then
		self.is_parent[self.parent[self.num]] = true
	end
	self.last_rendered[self.num] = -1
	self.refresh_interval[self.num] = options.refresh_interval or 60
	self.refresh_timer[self.num] = options.refresh_interval
	self.enabled[self.num] = options.enabled or 2
	self.canvas[self.num] = Image.createEmpty(self.width[self.num], self.height[self.num])
	self.render_function[self.num] = options.render_function
	return self.num
end


--Sys functions - not important ;)

function Element:render(id)
	local _parent = self.parent[id]
	if self.enabled[id] == 2 then
		if self.refresh_timer[id] >= self.refresh_interval[id] then
			self.last_rendered[id] = self.frame
			self.refresh_timer[id] = 0
			self.render_function[id](id)
		end
		if _parent > 0 then
			if self.last_rendered[_parent] ~= self.frame then
				Element:render(_parent)
			end
			self.last_rendered[_parent] = self.frame
			if not self.is_parent[id] then
				self.canvas[_parent]:blit(self.x_pos[id], self.y_pos[id], self.canvas[id])
			end
		end
		self.refresh_timer[id] = self.refresh_timer[id] + 1
	end
end
			





function Element:process()
	local _index
	for _index = self.num, 1, -1 do
		if not self.is_parent[_index] then
			Element:render(_index)
		end
	end
	for _index = self.num, 1, -1 do
		if self.is_parent[_index] then
			_parent = self.parent[_index]
			if _parent > 0 then
				self.canvas[_parent]:blit(self.x_pos[_index], self.y_pos[_index], self.canvas[_index])
			else
				screen:blit(self.x_pos[_index], self.y_pos[_index], self.canvas[_index])
			end
		end
	end
	self.frame = self.frame + 1
	if self.frame > 100000 then
		self.frame = 0
	end
	screen.waitVblankStart()
	screen.flip()
end