--[[

	BPDE API library

	Written by Bjrn Petter Kysnes (bepe86) for PSP OSi.

	You can contact the author (me) by mail (bk@cold-blade.com)

--]]

Keymapping = {
	num = 0,
	oldkeys = Controls.read(),
	key = { },
	event_function = { },
	enabled = { },
	hold = { }
}


function Keymapping:process()
	newkeys = Controls.read()
	for _index = 1, self.num do
		if self.enabled[_index] == true then
			if self.key[_index] == "analog" then
				dx = newkeys:analogX()
				dy = newkeys:analogY()
				if math.abs(dx) < 32 then
					dx = 0
				end
				if math.abs(dy) < 32 then
					dy = 0
				end
				self.event_function[_index](dx, dy)
			else
				state = newkeys:buttons() & self.key[_index] > 0
				if self.hold[_index] then
					self.event_function[_index](state)
				else
					if state and newkeys ~= self.oldkeys then
						self.event_function[_index]()
					end
				end
			end
		end
	end
	self.oldkeys = newkeys
end

function Keymapping:add(options)
	self.num = self.num + 1
	self.key[self.num] = options.key
	self.event_function[self.num] = options.event_function
	self.enabled[self.num] = options.enabled or true
	self.hold[self.num] = options.hold or false

	return self.num
end

Pointer = {
	input_id = {
		analog,
		fast_move
	},
	drawn = false,
	gui_id,
	image = Image.load("media/pointer.png"),
	speed = 1
}

function Pointer:add()
	self.gui_id = Element:add{x_pos = 50, y_pos = 50, width = 20, height = 20, parent = 1, render_function = Pointer_render, refresh_interval = 1}
	self.input_id.analog = Keymapping:add{event_function = Pointer_move, key = "analog"}
	self.input_id.fast_move = Keymapping:add{event_function = Pointer_fast, key = Controls.crossMask, hold = true}
end

function Pointer_render(id)
	if not Pointer.drawn then
		Pointer.drawn = true
		Element.canvas[id]:clear()
		Element.canvas[id]:blit(0, 0, Pointer.image)
	end
end

function Pointer_move(dx, dy)
	Element.x_pos[Pointer.gui_id] = Element.x_pos[Pointer.gui_id] + dx / 32 * Pointer.speed
	if Element.x_pos[Pointer.gui_id] < 0 then
		Element.x_pos[Pointer.gui_id] = 0
	elseif Element.x_pos[Pointer.gui_id] > RESOLUTION.X then
		Element.x_pos[Pointer.gui_id] = RESOLUTION.X
	end
	Element.y_pos[Pointer.gui_id] = Element.y_pos[Pointer.gui_id] + dy / 32 * Pointer.speed
	if Element.y_pos[Pointer.gui_id] < 0 then
		Element.y_pos[Pointer.gui_id] = 0
	elseif Element.y_pos[Pointer.gui_id] > RESOLUTION.Y then
		Element.y_pos[Pointer.gui_id] = RESOLUTION.Y
	end
end

function Pointer_fast(state)
	if state then
		Pointer.speed = 2
	else
		Pointer.speed = 1
	end
end